#!/usr/bin/env bash
#
# This script is deprecated and kept for backwards compatibility.
#
# This file is generated by l2tdevtools update-dependencies.py any dependency
# related changes should be made in dependencies.ini.

./config/linux/ubuntu_install_plaso.sh $1
