#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the event formatters manager."""

import unittest

from tests import test_lib as shared_test_lib


class FormattersManagerTest(shared_test_lib.BaseTestCase):
  """Tests for the event formatters manager."""

  # pylint: disable=protected-access

  # TODO: add tests for GetEventFormatterHelper
  # TODO: add tests for RegisterEventFormatterHelper
  # TODO: add tests for RegisterEventFormatterHelpers


if __name__ == '__main__':
  unittest.main()
